#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"

//#define RANGE 45
#define RANGE 60
//#define RANGE 90


#define FREESTYLE_ID "Microsoft SideWinder Freestyle Pro (USB)"
int joyNums;
int TargetJoystick;
SDL_Joystick *joy;

int HandleEvent(SDL_Event *event)
{
	int done;
	done=0;
	switch( event->type ) {
		case SDL_KEYDOWN:
			//printf("key '%s' pressed\n", SDL_GetKeyName(event->key.keysym.sym));
			if ( event->key.keysym.sym == SDLK_ESCAPE ) { done=1; }
			break;
		case SDL_QUIT:
			done=1;
			break;
	}
	return done;
}

int Do_sdl_event()
{
	int done;
	done=0;
	SDL_Event event;
	while( SDL_PollEvent(&event) ) {
		done=HandleEvent(&event);
	}
	return done;
}

int RunGLTest()
{
	Uint32 video_flags;
	int value;
	int bpp = 0;
	int rgb_size[3];
	int w = 640;
	int h = 480;
	int done = 0;
	GLenum gl_error;
	char* sdl_error;
	float cube[8][3]= {
		{ 0.5,  0.5, -0.5}, { 0.5, -0.5, -0.5}, {-0.5, -0.5, -0.5}, {-0.5,  0.5, -0.5},
		{-0.5,  0.5,  0.5}, { 0.5,  0.5,  0.5}, { 0.5, -0.5,  0.5}, {-0.5, -0.5,  0.5}  };
	
	putenv("SDL_VIDEODRIVER=directx");
	if( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) < 0 ) {
		fprintf(stderr,"Couldn't initialize SDL: %s\n",SDL_GetError());
		exit( 1 );
	}
	
	joy = SDL_JoystickOpen( TargetJoystick );
	
	SDL_ShowCursor(SDL_DISABLE);

	video_flags = SDL_OPENGL;
	if ( SDL_GetVideoInfo()->vfmt->BitsPerPixel <= 8 ) { bpp=8; } else { bpp=16; }
	switch (bpp) {
		case 8:  rgb_size[0]=3; rgb_size[1]=3; rgb_size[2]=2; break;
		case 16: rgb_size[0]=5; rgb_size[1]=5; rgb_size[2]=5; break;
		default: rgb_size[0]=8; rgb_size[1]=8; rgb_size[2]=8; break;
	}
	SDL_GL_SetAttribute( SDL_GL_RED_SIZE,   rgb_size[0] );
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, rgb_size[1] );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE,  rgb_size[2] );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	SDL_GL_SetAttribute( SDL_GL_MULTISAMPLEBUFFERS, 1 );
	SDL_GL_SetAttribute( SDL_GL_ACCELERATED_VISUAL, 1 );
	SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 1 );
	if ( SDL_SetVideoMode( w, h, bpp, video_flags ) == NULL ) {
		fprintf(stderr, "Couldn't set GL mode: %s\n", SDL_GetError());
		SDL_Quit(); exit(1);
	}

	SDL_WM_SetCaption( "SDL GL test", "testgl" );

	glViewport( 0, 0, w, h );
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );
	glOrtho( -2.0, 2.0, -2.0, 2.0, -20.0, 20.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glShadeModel(GL_SMOOTH);

	while( !done ) {
		float jx,jy;
		SDL_JoystickUpdate();
		if( SDL_JoystickNumAxes(joy) >= 1 ) {
			jx = SDL_JoystickGetAxis(joy,0) * RANGE / 32767;
			jy = SDL_JoystickGetAxis(joy,1) * RANGE / 32767;
		}
		
		glClearColor( 0.0, 0.0, 0.0, 1.0 );
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
		glMatrixMode(GL_MODELVIEW);
		
		glLoadIdentity(); // Matrix reset
		
		// Rotation
		glRotatef(5.0 , 0, 1.0, 0);
		glRotatef(2.0 , 1.0, 0, 0);
		glRotatef(jy, 1.0, 0, 0);
		glRotatef(-jx, 0, 0, 1.0);
	
		// draw CUBE
		glBegin( GL_QUADS );
		 glColor3f(1.0, 0.0, 0.0);
		 glVertex3fv(cube[0]);glVertex3fv(cube[1]);glVertex3fv(cube[2]);glVertex3fv(cube[3]);
		 glColor3f(0.0, 1.0, 0.0);
		 glVertex3fv(cube[3]);glVertex3fv(cube[4]);glVertex3fv(cube[7]);glVertex3fv(cube[2]);
		 glColor3f(0.0, 0.0, 1.0);
		 glVertex3fv(cube[0]);glVertex3fv(cube[5]);glVertex3fv(cube[6]);glVertex3fv(cube[1]);
		 glColor3f(0.0, 1.0, 1.0);
		 glVertex3fv(cube[5]);glVertex3fv(cube[4]);glVertex3fv(cube[7]);glVertex3fv(cube[6]);
		 glColor3f(1.0, 1.0, 0.0);
		 glVertex3fv(cube[5]);glVertex3fv(cube[0]);glVertex3fv(cube[3]);glVertex3fv(cube[4]);
		 glColor3f(1.0, 0.0, 1.0);
		 glVertex3fv(cube[6]);glVertex3fv(cube[1]);glVertex3fv(cube[2]);glVertex3fv(cube[7]);
		glEnd( );
		
		SDL_GL_SwapBuffers( );

		// GL error check ,  SDL error check
		gl_error = glGetError( );   sdl_error = SDL_GetError( );
		if( gl_error != GL_NO_ERROR ) {
			fprintf( stderr, "testgl: OpenGL error: %d\n", gl_error );
		}
		if( sdl_error[0] != '\0' ) {
			fprintf(stderr, "testgl: SDL error '%s'\n", sdl_error); SDL_ClearError();
		}
		
		// SDL event process
		done=Do_sdl_event();
		
		// wait
		SDL_Delay(10);
	}
	
	SDL_JoystickClose( joy );
	SDL_Quit();
	return 0;
}

void JoystickCheck(void)
{
	// Joystick check
	SDL_Init(SDL_INIT_JOYSTICK);
	joyNums = SDL_NumJoysticks();
	if( joyNums <= 0 ) {
		printf( "No joysticks were found.\n" );
		SDL_Quit(); exit(1);
	}
	TargetJoystick = -1;
	for( int i = 0; i < joyNums; i++ ) {
		printf( "%d, %s\n", i, SDL_JoystickName(i) );
		if (strcmp( SDL_JoystickName(i), FREESTYLE_ID )==0) {
			TargetJoystick = i;
			printf("Found. (%s)\n", SDL_JoystickName(i) );
		}
	}
	if (TargetJoystick == -1) {
		// Microsoft Sidewinder Freestyle Pro  is not found.
		printf( "FreeStylePro joystick is not found.\n" );
		TargetJoystick = 0; // first joystick
	}
}

int main(int argc, char *argv[])
{
	JoystickCheck();
	RunGLTest();
	
	return 0;
}
